@@ -1,21 +1,27 @@
-import grblas as gb
+from numbers import Integral, Number
+
 import numpy as np
 from numba import njit
-from numbers import Integral, Number
 from suitesparse_graphblas.utils import claim_buffer, claim_buffer_2d, unclaim_buffer
-from .. import ffi, lib
+
+import grblas as gb
+
+from .. import ffi, lib, monoid
 from ..base import call, record_raw
-from ..dtypes import lookup_dtype, INT64
+from ..dtypes import _INDEX, INT64, lookup_dtype
 from ..exceptions import check_status, check_status_carg
-from ..scalar import _CScalar
+from ..scalar import Scalar, _CScalar
 from ..utils import (
-    libget,
+    _CArray,
+    _Pointer,
     get_shape,
     ints_to_numpy_buffer,
+    libget,
     values_to_numpy_buffer,
     wrapdoc,
-    _CArray,
 )
+from .prefix_scan import prefix_scan
+from .scalar import gxb_scalar
 
 ffi_new = ffi.new
 
@@ -361,7 +367,7 @@ class ss:
         grblas.ss.diag
         Vector.ss.diag
         """
-        self._parent._expect_type(vector, gb.Vector, within="ss.diag", argname="vector")
+        vector = self._parent._expect_type(vector, gb.Vector, within="ss.diag", argname="vector")
         call("GxB_Matrix_diag", [self._parent, vector, _CScalar(k, dtype=INT64), None])
 
     def split(self, chunks, *, name=None):
@@ -462,6 +468,29 @@ class ss:
         m, n = _concat_mn(tiles)
         self._concat(tiles, m, n)
 
+    def build_scalar(self, rows, columns, value):
+        """
+        GxB_Matrix_build_Scalar
+
+        Like ``build``, but uses a scalar for all the values.
+
+        See Also
+        --------
+        Matrix.build
+        Matrix.from_values
+        """
+        rows = ints_to_numpy_buffer(rows, np.uint64, name="row indices")
+        columns = ints_to_numpy_buffer(columns, np.uint64, name="column indices")
+        if rows.size != columns.size:
+            raise ValueError(
+                f"`rows` and `columns` lengths must match: {rows.size}, {columns.size}"
+            )
+        scalar = gxb_scalar(self._parent.dtype, value)
+        status = lib.GxB_Matrix_build_Scalar(
+            self._parent._carg, _CArray(rows)._carg, _CArray(columns)._carg, scalar[0], rows.size
+        )
+        check_status(status, self._parent)
+
     def export(self, format=None, *, sort=False, give_ownership=False, raw=False):
         """
         GxB_Matrix_export_xxx
@@ -479,6 +508,7 @@ class ss:
                 - "bitmapc"
                 - "fullr"
                 - "fullc"
+                - "coo"
         sort : bool, default False
             Whether to sort indices if the format is "csr", "csc", "hypercsr", or "hypercsc".
         give_ownership : bool, default False
@@ -507,7 +537,7 @@ class ss:
                 - indptr : ndarray(dtype=uint64, ndim=1, size=nrows + 1)
                 - col_indices : ndarray(dtype=uint64, ndim=1, size=nvals)
                 - values : ndarray(ndim=1, size=nvals)
-                - sorted_index : bool
+                - sorted_cols : bool
                     - True if the values in "col_indices" are sorted
                 - nrows : int
                 - ncols : int
@@ -515,7 +545,7 @@ class ss:
                 - indptr : ndarray(dtype=uint64, ndim=1, size=ncols + 1)
                 - row_indices : ndarray(dtype=uint64, ndim=1, size=nvals)
                 - values : ndarray(ndim=1, size=nvals)
-                - sorted_index : bool
+                - sorted_rows : bool
                     - True if the values in "row_indices" are sorted
                 - nrows : int
                 - ncols : int
@@ -524,7 +554,7 @@ class ss:
                 - rows : ndarray(dtype=uint64, ndim=1, size=nvec)
                 - col_indices : ndarray(dtype=uint64, ndim=1, size=nvals)
                 - values : ndarray(ndim=1, size=nvals)
-                - sorted_index : bool
+                - sorted_cols : bool
                     - True if the values in "col_indices" are sorted
                 - nrows : int
                 - ncols : int
@@ -535,7 +565,7 @@ class ss:
                 - cols : ndarray(dtype=uint64, ndim=1, size=nvec)
                 - row_indices : ndarray(dtype=uint64, ndim=1, size=nvals)
                 - values : ndarray(ndim=1, size=nvals)
-                - sorted_index : bool
+                - sorted_rows : bool
                     - True if the values in "row_indices" are sorted
                 - nrows : int
                 - ncols : int
@@ -595,6 +625,17 @@ class ss:
                         - Stored row-oriented
                     - nrows : int
                     - ncols : int
+            - "coo" format
+                - rows : ndarray(dtype=uint64, ndim=1, size=nvals)
+                - cols : ndarray(dtype=uint64, ndim=1, size=nvals)
+                - values : ndarray(ndim=1, size=nvals)
+                - nrows : int
+                - ncols : int
+                - ``sorted=True``
+                    - sorted_rows : True, only present if stored row-wise
+                        - Indices are sorted by row then by column.
+                    - sorted_columns : True, only present if stored column-wise
+                        - Indices are sorted by column then by row.
 
         Examples
         --------
@@ -619,20 +660,70 @@ class ss:
         return self._export(format, sort=sort, raw=raw, give_ownership=True, method="unpack")
 
     def _export(self, format=None, *, sort=False, give_ownership=False, raw=False, method):
-        if give_ownership:
+        if format is None:
+            format = self.format
+        else:
+            format = format.lower()
+        if give_ownership or format == "coo":
             parent = self._parent
         else:
             parent = self._parent.dup(name=f"M_{method}")
         dtype = np.dtype(parent.dtype.np_type)
         index_dtype = np.dtype(np.uint64)
 
-        if format is None:
-            format = self.format
-        else:
-            format = format.lower()
-
         nrows = parent._nrows
         ncols = parent._ncols
+        if format == "coo":
+            if sort:
+                # It's weird, but waiting makes values sorted (according to the storage orientation)
+                # If we don't wait, we don't know whether the values are sorted or not.
+                parent.wait()
+            if self.is_iso:
+                # Should we expose a way to do `to_values` without values?
+                # Passing NULL for values is SuiteSparse-specific.
+                nvals = parent._nvals
+                rows = _CArray(size=nvals, name="&rows_array")
+                columns = _CArray(size=nvals, name="&columns_array")
+                n = ffi_new("GrB_Index*")
+                scalar = Scalar(n, _INDEX, name="s_nvals", empty=True)
+                scalar.value = nvals
+                call(
+                    f"GrB_Matrix_extractTuples_{parent.dtype.name}",
+                    [rows, columns, None, _Pointer(scalar), parent],
+                )
+                value = parent.reduce_scalar(gb.monoid.any).value
+                rv = {
+                    "format": "coo",
+                    "nrows": nrows,
+                    "ncols": ncols,
+                    "rows": rows.array,
+                    "cols": columns.array,
+                    "values": np.array([value], dtype=dtype),
+                    "is_iso": True,
+                }
+            else:
+                rows, columns, values = parent.to_values()
+                rv = {
+                    "format": "coo",
+                    "nrows": nrows,
+                    "ncols": ncols,
+                    "rows": rows,
+                    "cols": columns,
+                    "values": values,
+                }
+            if sort:
+                if self.format[-1] == "r":
+                    rv["sorted_rows"] = True
+                else:
+                    rv["sorted_cols"] = True
+            if give_ownership:
+                if method == "export":
+                    parent.__del__()
+                    parent.gb_obj = ffi.NULL
+                else:
+                    parent.clear()
+            return rv
+
         if method == "export":
             mhandle = ffi_new("GrB_Matrix*", parent._carg)
             type_ = ffi_new("GrB_Type*")
@@ -690,7 +781,7 @@ class ss:
             rv = {
                 "indptr": indptr,
                 "col_indices": col_indices,
-                "sorted_index": True if sort else not jumbled[0],
+                "sorted_cols": True if sort else not jumbled[0],
                 "nrows": nrows,
                 "ncols": ncols,
             }
@@ -732,7 +823,7 @@ class ss:
             rv = {
                 "indptr": indptr,
                 "row_indices": row_indices,
-                "sorted_index": True if sort else not jumbled[0],
+                "sorted_rows": True if sort else not jumbled[0],
                 "nrows": nrows,
                 "ncols": ncols,
             }
@@ -785,7 +876,7 @@ class ss:
                 "indptr": indptr,
                 "rows": rows,
                 "col_indices": col_indices,
-                "sorted_index": True if sort else not jumbled[0],
+                "sorted_cols": True if sort else not jumbled[0],
                 "nrows": nrows,
                 "ncols": ncols,
             }
@@ -840,7 +931,7 @@ class ss:
                 "indptr": indptr,
                 "cols": cols,
                 "row_indices": row_indices,
-                "sorted_index": True if sort else not jumbled[0],
+                "sorted_rows": True if sort else not jumbled[0],
                 "nrows": nrows,
                 "ncols": ncols,
             }
@@ -954,7 +1045,7 @@ class ss:
         values,
         col_indices,
         is_iso=False,
-        sorted_index=False,
+        sorted_cols=False,
         take_ownership=False,
         dtype=None,
         format=None,
@@ -975,7 +1066,7 @@ class ss:
         is_iso : bool, default False
             Is the Matrix iso-valued (meaning all the same value)?
             If true, then `values` should be a length 1 array.
-        sorted_index : bool, default False
+        sorted_cols : bool, default False
             Indicate whether the values in "col_indices" are sorted.
         take_ownership : bool, default False
             If True, perform a zero-copy data transfer from input numpy arrays
@@ -1009,7 +1100,7 @@ class ss:
             values=values,
             col_indices=col_indices,
             is_iso=is_iso,
-            sorted_index=sorted_index,
+            sorted_cols=sorted_cols,
             take_ownership=take_ownership,
             dtype=dtype,
             format=format,
@@ -1024,7 +1115,7 @@ class ss:
         values,
         col_indices,
         is_iso=False,
-        sorted_index=False,
+        sorted_cols=False,
         take_ownership=False,
         format=None,
         **ignored_kwargs,
@@ -1042,7 +1133,7 @@ class ss:
             values=values,
             col_indices=col_indices,
             is_iso=is_iso,
-            sorted_index=sorted_index,
+            sorted_cols=sorted_cols,
             take_ownership=take_ownership,
             format=format,
             method="pack",
@@ -1059,7 +1150,7 @@ class ss:
         values,
         col_indices,
         is_iso=False,
-        sorted_index=False,
+        sorted_cols=False,
         take_ownership=False,
         dtype=None,
         format=None,
@@ -1100,7 +1191,7 @@ class ss:
             col_indices.nbytes,
             values.nbytes,
             is_iso,
-            not sorted_index,
+            not sorted_cols,
             ffi.NULL,
         )
         if method == "import":
@@ -1129,7 +1220,7 @@ class ss:
         values,
         row_indices,
         is_iso=False,
-        sorted_index=False,
+        sorted_rows=False,
         take_ownership=False,
         dtype=None,
         format=None,
@@ -1150,7 +1241,7 @@ class ss:
         is_iso : bool, default False
             Is the Matrix iso-valued (meaning all the same value)?
             If true, then `values` should be a length 1 array.
-        sorted_index : bool, default False
+        sorted_rows : bool, default False
             Indicate whether the values in "row_indices" are sorted.
         take_ownership : bool, default False
             If True, perform a zero-copy data transfer from input numpy arrays
@@ -1184,7 +1275,7 @@ class ss:
             values=values,
             row_indices=row_indices,
             is_iso=is_iso,
-            sorted_index=sorted_index,
+            sorted_rows=sorted_rows,
             take_ownership=take_ownership,
             dtype=dtype,
             format=format,
@@ -1199,7 +1290,7 @@ class ss:
         values,
         row_indices,
         is_iso=False,
-        sorted_index=False,
+        sorted_rows=False,
         take_ownership=False,
         format=None,
         **ignored_kwargs,
@@ -1217,7 +1308,7 @@ class ss:
             values=values,
             row_indices=row_indices,
             is_iso=is_iso,
-            sorted_index=sorted_index,
+            sorted_rows=sorted_rows,
             take_ownership=take_ownership,
             format=format,
             method="pack",
@@ -1234,7 +1325,7 @@ class ss:
         values,
         row_indices,
         is_iso=False,
-        sorted_index=False,
+        sorted_rows=False,
         take_ownership=False,
         dtype=None,
         format=None,
@@ -1275,7 +1366,7 @@ class ss:
             row_indices.nbytes,
             values.nbytes,
             is_iso,
-            not sorted_index,
+            not sorted_rows,
             ffi.NULL,
         )
         if method == "import":
@@ -1306,7 +1397,7 @@ class ss:
         col_indices,
         nvec=None,
         is_iso=False,
-        sorted_index=False,
+        sorted_cols=False,
         take_ownership=False,
         dtype=None,
         format=None,
@@ -1331,7 +1422,7 @@ class ss:
         is_iso : bool, default False
             Is the Matrix iso-valued (meaning all the same value)?
             If true, then `values` should be a length 1 array.
-        sorted_index : bool, default False
+        sorted_cols : bool, default False
             Indicate whether the values in "col_indices" are sorted.
         take_ownership : bool, default False
             If True, perform a zero-copy data transfer from input numpy arrays
@@ -1367,7 +1458,7 @@ class ss:
             col_indices=col_indices,
             nvec=nvec,
             is_iso=is_iso,
-            sorted_index=sorted_index,
+            sorted_cols=sorted_cols,
             take_ownership=take_ownership,
             dtype=dtype,
             format=format,
@@ -1384,7 +1475,7 @@ class ss:
         col_indices,
         nvec=None,
         is_iso=False,
-        sorted_index=False,
+        sorted_cols=False,
         take_ownership=False,
         format=None,
         **ignored_kwargs,
@@ -1404,7 +1495,7 @@ class ss:
             col_indices=col_indices,
             nvec=nvec,
             is_iso=is_iso,
-            sorted_index=sorted_index,
+            sorted_cols=sorted_cols,
             take_ownership=take_ownership,
             format=format,
             method="pack",
@@ -1423,7 +1514,7 @@ class ss:
         col_indices,
         nvec=None,
         is_iso=False,
-        sorted_index=False,
+        sorted_cols=False,
         take_ownership=False,
         dtype=None,
         format=None,
@@ -1471,7 +1562,7 @@ class ss:
             values.nbytes,
             is_iso,
             nvec,
-            not sorted_index,
+            not sorted_cols,
             ffi.NULL,
         )
         if method == "import":
@@ -1503,7 +1594,7 @@ class ss:
         row_indices,
         nvec=None,
         is_iso=False,
-        sorted_index=False,
+        sorted_rows=False,
         take_ownership=False,
         dtype=None,
         format=None,
@@ -1527,7 +1618,7 @@ class ss:
         is_iso : bool, default False
             Is the Matrix iso-valued (meaning all the same value)?
             If true, then `values` should be a length 1 array.
-        sorted_index : bool, default False
+        sorted_rows : bool, default False
             Indicate whether the values in "row_indices" are sorted.
         take_ownership : bool, default False
             If True, perform a zero-copy data transfer from input numpy arrays
@@ -1563,7 +1654,7 @@ class ss:
             row_indices=row_indices,
             nvec=nvec,
             is_iso=is_iso,
-            sorted_index=sorted_index,
+            sorted_rows=sorted_rows,
             take_ownership=take_ownership,
             dtype=dtype,
             format=format,
@@ -1580,7 +1671,7 @@ class ss:
         row_indices,
         nvec=None,
         is_iso=False,
-        sorted_index=False,
+        sorted_rows=False,
         take_ownership=False,
         format=None,
         **ignored_kwargs,
@@ -1600,7 +1691,7 @@ class ss:
             row_indices=row_indices,
             nvec=nvec,
             is_iso=is_iso,
-            sorted_index=sorted_index,
+            sorted_rows=sorted_rows,
             take_ownership=take_ownership,
             format=format,
             method="pack",
@@ -1619,7 +1710,7 @@ class ss:
         row_indices,
         nvec=None,
         is_iso=False,
-        sorted_index=False,
+        sorted_rows=False,
         take_ownership=False,
         dtype=None,
         format=None,
@@ -1667,7 +1758,7 @@ class ss:
             values.nbytes,
             is_iso,
             nvec,
-            not sorted_index,
+            not sorted_rows,
             ffi.NULL,
         )
         if method == "import":
@@ -2221,7 +2312,6 @@ class ss:
             The number of columns for the Matrix.
             If not provided, will be inferred from values if it is 2d.
         is_iso : bool, default False
-            Not yet supported.
             Is the Matrix iso-valued (meaning all the same value)?
             If true, then `values` should be a length 1 array.
         take_ownership : bool, default False
@@ -2341,6 +2431,135 @@ class ss:
         unclaim_buffer(values)
         return matrix
 
+    @classmethod
+    def import_coo(
+        cls,
+        rows,
+        cols,
+        values,
+        *,
+        nrows,
+        ncols,
+        is_iso=False,
+        sorted_rows=False,
+        sorted_cols=False,
+        take_ownership=False,
+        dtype=None,
+        format=None,
+        name=None,
+    ):
+        """
+        GrB_Matrix_build_XXX and GxB_Matrix_build_Scalar
+
+        Create a new Matrix from indices and values in coordinate format.
+
+        Parameters
+        ----------
+        rows : array-like
+        cols : array-likd
+        values : array-like
+        nrows : int
+            The number of rows for the Matrix.
+        ncols : int
+            The number of columns for the Matrix.
+        is_iso : bool, default False
+            Is the Matrix iso-valued (meaning all the same value)?
+            If true, then `values` should be a length 1 array.
+        sorted_rows : bool, default False
+            Ignored.  True indicates indices are sorted by row, then column.
+        sorted_cols : bool, default False
+            Ignored.  True indicates indices are sorted by column, then row.
+        take_ownership : bool, default False
+            Ignored.  Zero-copy is not possible for "coo" format.
+        dtype : dtype, optional
+            dtype of the new Matrix.
+            If not specified, this will be inferred from `values`.
+        format : str, optional
+            Must be "coo" or None.  This is included to be compatible with
+            the dict returned from exporting.
+        name : str, optional
+            Name of the new Matrix.
+
+        Returns
+        -------
+        Matrix
+        """
+        return cls._import_coo(
+            rows=rows,
+            cols=cols,
+            values=values,
+            nrows=nrows,
+            ncols=ncols,
+            is_iso=is_iso,
+            take_ownership=take_ownership,
+            dtype=dtype,
+            format=format,
+            name=name,
+            method="import",
+        )
+
+    def pack_coo(
+        self,
+        rows,
+        cols,
+        values,
+        *,
+        is_iso=False,
+        sorted_rows=False,
+        sorted_cols=False,
+        take_ownership=False,
+        format=None,
+        **unused_kwargs,
+    ):
+        """
+        GrB_Matrix_build_XXX and GxB_Matrix_build_Scalar
+
+        `pack_coo` is like `import_coo` except it "packs" data into an
+        existing Matrix.  This is the opposite of ``unpack("coo")``
+
+        See `Matrix.ss.import_coo` documentation for more details.
+        """
+        return self._import_coo(
+            rows=rows,
+            cols=cols,
+            values=values,
+            is_iso=is_iso,
+            take_ownership=take_ownership,
+            format=format,
+            method="pack",
+            matrix=self._parent,
+        )
+
+    @classmethod
+    def _import_coo(
+        cls,
+        rows,
+        cols,
+        values,
+        *,
+        nrows=None,
+        ncols=None,
+        is_iso=False,
+        take_ownership=False,
+        dtype=None,
+        format=None,
+        name=None,
+        method,
+        matrix=None,
+    ):
+        if format is not None and format.lower() != "coo":
+            raise ValueError(f"Invalid format: {format!r}.  Must be None or 'coo'.")
+        if method == "pack":
+            dtype = matrix.dtype
+        values, dtype = values_to_numpy_buffer(values, dtype)
+        if method == "import":
+            matrix = gb.Matrix.new(dtype, nrows=nrows, ncols=ncols, name=name)
+        if is_iso:
+            matrix.ss.build_scalar(rows, cols, values[0])
+        else:
+            matrix.build(rows, cols, values)
+        return matrix
+
     @classmethod
     def import_any(
         cls,
@@ -2356,14 +2575,15 @@ class ss:
         name=None,
         # CSR/CSC/HyperCSR/HyperCSC
         indptr=None,
-        sorted_index=False,
         # CSR/HyperCSR
         col_indices=None,
-        # HyperCSR
+        sorted_cols=False,
+        # HyperCSR/COO
         rows=None,
         # CSC/HyperCSC
         row_indices=None,
-        # HyperCSC
+        sorted_rows=False,
+        # HyperCSC/COO
         cols=None,
         # HyperCSR/HyperCSC
         nvec=None,  # optional
@@ -2412,14 +2632,15 @@ class ss:
             name=name,
             # CSR/CSC/HyperCSR/HyperCSC
             indptr=indptr,
-            sorted_index=sorted_index,
             # CSR/HyperCSR
             col_indices=col_indices,
-            # HyperCSR
+            sorted_cols=sorted_cols,
+            # HyperCSR/COO
             rows=rows,
             # CSC/HyperCSC
             row_indices=row_indices,
-            # HyperCSC
+            sorted_rows=sorted_rows,
+            # HyperCSC/COO
             cols=cols,
             # HyperCSR/HyperCSC
             nvec=nvec,
@@ -2439,14 +2660,15 @@ class ss:
         format=None,
         # CSR/CSC/HyperCSR/HyperCSC
         indptr=None,
-        sorted_index=False,
         # CSR/HyperCSR
         col_indices=None,
-        # HyperCSR
+        sorted_cols=False,
+        # HyperCSR/COO
         rows=None,
         # CSC/HyperCSC
         row_indices=None,
-        # HyperCSC
+        sorted_rows=False,
+        # HyperCSC/COO
         cols=None,
         # HyperCSR/HyperCSC
         nvec=None,  # optional
@@ -2474,14 +2696,15 @@ class ss:
             format=format,
             # CSR/CSC/HyperCSR/HyperCSC
             indptr=indptr,
-            sorted_index=sorted_index,
             # CSR/HyperCSR
             col_indices=col_indices,
-            # HyperCSR
+            sorted_cols=sorted_cols,
+            # HyperCSR/COO
             rows=rows,
             # CSC/HyperCSC
             row_indices=row_indices,
-            # HyperCSC
+            sorted_rows=sorted_rows,
+            # HyperCSC/COO
             cols=cols,
             # HyperCSR/HyperCSC
             nvec=nvec,
@@ -2507,14 +2730,15 @@ class ss:
         name=None,
         # CSR/CSC/HyperCSR/HyperCSC
         indptr=None,
-        sorted_index=False,
         # CSR/HyperCSR
         col_indices=None,
-        # HyperCSR
+        sorted_cols=False,
+        # HyperCSR/COO
         rows=None,
         # CSC/HyperCSC
         row_indices=None,
-        # HyperCSC
+        sorted_rows=False,
+        # HyperCSC/COO
         cols=None,
         # HyperCSR/HyperCSC
         nvec=None,  # optional
@@ -2573,6 +2797,10 @@ class ss:
                 # Then default to row-oriented
                 if format is None:
                     format = "bitmapr"
+            elif rows is not None or cols is not None:
+                if rows is None or cols is None:
+                    raise ValueError("coo requires both `rows` and `cols`")
+                format = "coo"
             else:
                 if (
                     isinstance(values, np.ndarray)
@@ -2596,7 +2824,7 @@ class ss:
                 values=values,
                 col_indices=col_indices,
                 is_iso=is_iso,
-                sorted_index=sorted_index,
+                sorted_cols=sorted_cols,
                 take_ownership=take_ownership,
                 dtype=dtype,
                 name=name,
@@ -2609,7 +2837,7 @@ class ss:
                 values=values,
                 row_indices=row_indices,
                 is_iso=is_iso,
-                sorted_index=sorted_index,
+                sorted_rows=sorted_rows,
                 take_ownership=take_ownership,
                 dtype=dtype,
                 name=name,
@@ -2624,7 +2852,7 @@ class ss:
                 values=values,
                 col_indices=col_indices,
                 is_iso=is_iso,
-                sorted_index=sorted_index,
+                sorted_cols=sorted_cols,
                 take_ownership=take_ownership,
                 dtype=dtype,
                 name=name,
@@ -2639,7 +2867,7 @@ class ss:
                 values=values,
                 row_indices=row_indices,
                 is_iso=is_iso,
-                sorted_index=sorted_index,
+                sorted_rows=sorted_rows,
                 take_ownership=take_ownership,
                 dtype=dtype,
                 name=name,
@@ -2688,9 +2916,45 @@ class ss:
                 dtype=dtype,
                 name=name,
             )
+        elif format == "coo":
+            return getattr(obj, f"{method}_coo")(
+                nrows=nrows,
+                ncols=ncols,
+                rows=rows,
+                cols=cols,
+                values=values,
+                is_iso=is_iso,
+                take_ownership=take_ownership,
+                dtype=dtype,
+                name=name,
+            )
         else:
             raise ValueError(f"Invalid format: {format}")
 
     @wrapdoc(head)
     def head(self, n=10, *, sort=False, dtype=None):
         return head(self._parent, n, sort=sort, dtype=dtype)
+
+    def scan_columns(self, op=monoid.plus, *, name=None):
+        """Perform a prefix scan across columns with the given monoid.
+
+        For example, use `monoid.plus` (the default) to perform a cumulative sum,
+        and `monoid.times` for cumulative product.  Works with any monoid.
+
+        Returns
+        -------
+        Vector
+        """
+        return prefix_scan(self._parent.T, op, name=name, within="scan_columns")
+
+    def scan_rows(self, op=monoid.plus, *, name=None):
+        """Perform a prefix scan across rows with the given monoid.
+
+        For example, use `monoid.plus` (the default) to perform a cumulative sum,
+        and `monoid.times` for cumulative product.  Works with any monoid.
+
+        Returns
+        -------
+        Vector
+        """
+        return prefix_scan(self._parent, op, name=name, within="scan_rows")
