@@ -1,13 +1,23 @@
 import numpy as np
-import grblas as gb
 from numba import njit
 from suitesparse_graphblas.utils import claim_buffer, unclaim_buffer
-from .. import ffi, lib
+
+import grblas as gb
+
+from .. import ffi, lib, monoid
 from ..base import call
-from ..dtypes import lookup_dtype, INT64
+from ..dtypes import INT64, lookup_dtype
 from ..exceptions import check_status, check_status_carg
 from ..scalar import _CScalar
-from ..utils import libget, ints_to_numpy_buffer, values_to_numpy_buffer, wrapdoc
+from ..utils import (
+    _CArray,
+    ints_to_numpy_buffer,
+    libget,
+    values_to_numpy_buffer,
+    wrapdoc,
+)
+from .prefix_scan import prefix_scan
+from .scalar import gxb_scalar
 
 ffi_new = ffi.new
 
@@ -128,7 +138,7 @@ class ss:
         Matrix.ss.diag
 
         """
-        self._parent._expect_type(
+        matrix = self._parent._expect_type(
             matrix, (gb.Matrix, gb.matrix.TransposedMatrix), within="ss.diag", argname="matrix"
         )
         if type(matrix) is gb.matrix.TransposedMatrix:
@@ -137,6 +147,24 @@ class ss:
             matrix = matrix._matrix
         call("GxB_Vector_diag", [self._parent, matrix, _CScalar(k, dtype=INT64), None])
 
+    def build_scalar(self, indices, value):
+        """
+        GxB_Vector_build_Scalar
+
+        Like ``build``, but uses a scalar for all the values.
+
+        See Also
+        --------
+        Vector.build
+        Vector.from_values
+        """
+        indices = ints_to_numpy_buffer(indices, np.uint64, name="indices")
+        scalar = gxb_scalar(self._parent.dtype, value)
+        status = lib.GxB_Vector_build_Scalar(
+            self._parent._carg, _CArray(indices)._carg, scalar[0], indices.size
+        )
+        check_status(status, self._parent)
+
     def export(self, format=None, *, sort=False, give_ownership=False, raw=False):
         """
         GxB_Vextor_export_xxx
@@ -1007,3 +1035,15 @@ class ss:
     @wrapdoc(head)
     def head(self, n=10, *, sort=False, dtype=None):
         return head(self._parent, n, sort=sort, dtype=dtype)
+
+    def scan(self, op=monoid.plus, *, name=None):
+        """Perform a prefix scan with the given monoid.
+
+        For example, use `monoid.plus` (the default) to perform a cumulative sum,
+        and `monoid.times` for cumulative product.  Works with any monoid.
+
+        Returns
+        -------
+        Scalar
+        """
+        return prefix_scan(self._parent, op, name=name, within="scan")
