@@ -1,18 +1,19 @@
 from . import Matrix, Vector
-from .dtypes import lookup_dtype, FP64
+from .dtypes import FP64, lookup_dtype
 from .exceptions import GrblasException
 from .matrix import TransposedMatrix
+from .utils import output_type
 
 
-def draw(m):
+def draw(m):  # pragma: no cover
     try:
         import networkx as nx
-    except ImportError:  # pragma: no cover
+    except ImportError:
         print("`draw` requires networkx to be installed")
         return
     try:
         import matplotlib.pyplot as plt
-    except ImportError:  # pragma: no cover
+    except ImportError:
         print("`draw` requires matplotlib to be installed")
         return
 
@@ -53,7 +54,7 @@ def from_numpy(m):
         raise GrblasException("m.ndim must be <= 2")
 
     try:
-        from scipy.sparse import csr_matrix, coo_matrix
+        from scipy.sparse import coo_matrix, csr_matrix
     except ImportError:  # pragma: no cover
         raise ImportError("scipy is required to import from numpy")
 
@@ -96,7 +97,7 @@ def to_numpy(m):
         import scipy  # noqa
     except ImportError:  # pragma: no cover
         raise ImportError("scipy is required to export to numpy")
-    if type(m) is Vector:
+    if output_type(m) is Vector:
         return to_scipy_sparse_matrix(m).toarray()[0]
     else:
         sparse = to_scipy_sparse_matrix(m, "coo")
@@ -110,12 +111,12 @@ def to_scipy_sparse_matrix(m, format="csr"):
     import scipy.sparse as ss
 
     format = format.lower()
-    if type(m) is Vector:
+    if output_type(m) is Vector:
         indices, data = m.to_values()
         if format == "csc":
-            return ss.csc_matrix((data, indices, [0, len(data)]))
+            return ss.csc_matrix((data, indices, [0, len(data)]), shape=(m.size, 1))
         else:
-            rv = ss.csr_matrix((data, indices, [0, len(data)]))
+            rv = ss.csr_matrix((data, indices, [0, len(data)]), shape=(1, m.size))
             if format == "csr":
                 return rv
     else:
