@@ -1,16 +1,18 @@
 import inspect
 import itertools
 import re
-import numpy as np
-import numba
 from collections.abc import Mapping
 from functools import lru_cache
 from types import FunctionType, ModuleType
-from . import ffi, lib, unary, binary, monoid, semiring, op
-from .dtypes import lookup_dtype, unify, INT8, _sample_values, _supports_complex
+
+import numba
+import numpy as np
+
+from . import binary, ffi, lib, monoid, op, semiring, unary
+from .dtypes import INT8, _sample_values, _supports_complex, lookup_dtype, unify
 from .exceptions import UdfParseError, check_status_carg
 from .expr import InfixExprBase
-from .utils import libget
+from .utils import libget, output_type
 
 ffi_new = ffi.new
 UNKNOWN_OPCLASS = "UnknownOpClass"
@@ -45,12 +47,12 @@ def _call_op(op, left, right=None, **kwargs):
         )
 
     # op(A, 1) -> apply (or select once available)
-    from .vector import Vector
     from .matrix import Matrix, TransposedMatrix
+    from .vector import Vector
 
-    if type(left) in {Vector, Matrix, TransposedMatrix}:
+    if output_type(left) in {Vector, Matrix, TransposedMatrix}:
         return left.apply(op, right=right, **kwargs)
-    elif type(right) in {Vector, Matrix, TransposedMatrix}:
+    elif output_type(right) in {Vector, Matrix, TransposedMatrix}:
         return right.apply(op, left=left, **kwargs)
     raise TypeError(
         f"Bad types when calling {op!r}.  Got types: {type(left)}, {type(right)}.\n"
@@ -88,10 +90,10 @@ class TypedBuiltinUnaryOp(TypedOpBase):
     opclass = "UnaryOp"
 
     def __call__(self, val):
-        from .vector import Vector
         from .matrix import Matrix, TransposedMatrix
+        from .vector import Vector
 
-        if type(val) in {Vector, Matrix, TransposedMatrix}:
+        if output_type(val) in {Vector, Matrix, TransposedMatrix}:
             return val.apply(self)
         raise TypeError(
             f"Bad type when calling {self!r}.\n"
@@ -654,6 +656,7 @@ class UnaryOp(OpBase):
         module, funcname = cls._remove_nesting(name, module=op, modname="op", strict=False)
         if not hasattr(module, funcname):
             setattr(module, funcname, unary_op)
+        return unary_op
 
     @classmethod
     def _initialize(cls):
@@ -855,6 +858,7 @@ class BinaryOp(OpBase):
         module, funcname = cls._remove_nesting(name, module=op, modname="op", strict=False)
         if not hasattr(module, funcname):
             setattr(module, funcname, binary_op)
+        return binary_op
 
     @classmethod
     def _initialize(cls):
@@ -888,6 +892,9 @@ class BinaryOp(OpBase):
         # cdiv truncates towards 0, while floordiv truncates towards -inf
         BinaryOp.register_new("floordiv", lambda x, y: x // y)
 
+        # For aggregators
+        BinaryOp.register_new("absfirst", lambda x, y: abs(x))
+
         def isclose(rel_tol=1e-7, abs_tol=0.0):
             def inner(x, y):
                 return x == y or abs(x - y) <= max(rel_tol * max(abs(x), abs(y)), abs_tol)
@@ -1035,6 +1042,7 @@ class Monoid(OpBase):
         module, funcname = cls._remove_nesting(name, module=op, modname="op", strict=False)
         if not hasattr(module, funcname):
             setattr(module, funcname, monoid)
+        return monoid
 
     def __init__(self, name, binaryop=None, *, anonymous=False):
         super().__init__(name, anonymous=anonymous)
@@ -1070,6 +1078,7 @@ class Monoid(OpBase):
                 cur_op._typed_ops["BOOL"] = typed_op
 
         for cur_op in (monoid.lor, monoid.land, monoid.lxnor, monoid.lxor):
+            bool_op = cur_op._typed_ops["BOOL"]
             for dtype in (
                 "FP32",
                 "FP64",
@@ -1082,10 +1091,11 @@ class Monoid(OpBase):
                 "UINT32",
                 "UINT64",
             ):
-                assert dtype not in cur_op.types
+                if dtype in cur_op.types:  # pragma: no cover
+                    continue
                 cur_op.types[dtype] = "BOOL"
                 cur_op.coercions[dtype] = "BOOL"
-                cur_op._typed_ops[dtype] = lor
+                cur_op._typed_ops[dtype] = bool_op
 
     __call__ = TypedBuiltinMonoid.__call__
 
@@ -1138,7 +1148,7 @@ class Semiring(OpBase):
         if type(binaryop) is not BinaryOp:
             raise TypeError(f"binaryop must be a BinaryOp, not {type(binaryop)}")
         if name is None:
-            name = f"{monoid.name}_{binaryop.name}"
+            name = f"{monoid.name}_{binaryop.name}".replace(".", "_")
         new_type_obj = cls(name, monoid, binaryop, anonymous=anonymous)
         for binary_in, binary_func in binaryop._typed_ops.items():
             binary_out = binary_func.return_type
@@ -1184,6 +1194,7 @@ class Semiring(OpBase):
         module, funcname = cls._remove_nesting(name, module=op, modname="op", strict=False)
         if not hasattr(module, funcname):
             setattr(module, funcname, semiring)
+        return semiring
 
     @classmethod
     def _initialize(cls):
@@ -1214,6 +1225,10 @@ class Semiring(OpBase):
         for orig_name, orig in div_semirings.items():
             cls.register_new(f"{orig_name[:-3]}truediv", orig.monoid, binary.truediv)
             cls.register_new(f"{orig_name[:-3]}floordiv", orig.monoid, binary.floordiv)
+        # For aggregators
+        cls.register_new("plus_pow", monoid.plus, binary.pow)
+        cls.register_new("plus_absfirst", monoid.plus, binary.absfirst)
+        cls.register_new("max_absfirst", monoid.max, binary.absfirst)
 
         # Update type information with sane coercion
         for lname in ("any", "eq", "land", "lor", "lxnor", "lxor"):
@@ -1292,6 +1307,27 @@ class Semiring(OpBase):
                             op._typed_ops[dtype] = typed_op
                             op.coercions[dtype] = target_type
 
+        # Handle a few boolean cases
+        for opname, targetname in (
+            ("max_first", "lor_first"),
+            ("max_second", "lor_second"),
+            ("max_land", "lor_land"),
+            ("max_lor", "lor_lor"),
+            ("max_lxor", "lor_lxor"),
+            ("min_first", "land_first"),
+            ("min_second", "land_second"),
+            ("min_land", "land_land"),
+            ("min_lor", "land_lor"),
+            ("min_lxor", "land_lxor"),
+        ):
+            op = getattr(semiring, opname)
+            target = getattr(semiring, targetname)
+            if "BOOL" in op.types or "BOOL" not in target.types:  # pragma: no cover
+                continue
+            op.types["BOOL"] = target.types["BOOL"]
+            op._typed_ops["BOOL"] = target._typed_ops["BOOL"]
+            op.coercions["BOOL"] = "BOOL"
+
     def __init__(self, name, monoid=None, binaryop=None, *, anonymous=False):
         super().__init__(name, anonymous=anonymous)
         self._monoid = monoid
@@ -1324,6 +1360,10 @@ def get_typed_op(op, dtype, dtype2=None):
         return get_typed_op(op, dtype, dtype2)
     elif isinstance(op, TypedOpBase):
         return op
+    elif isinstance(op, Aggregator):
+        return op[dtype]
+    elif isinstance(op, TypedAggregator):
+        return op
     else:
         raise TypeError(f"Unable to get typed operator from object with type {type(op)}")
 
@@ -1341,8 +1381,68 @@ def find_opclass(gb_op):
     return gb_op, opclass
 
 
+def get_semiring(monoid, binaryop):
+    """Get or create a Semiring object from a monoid and binaryop.
+
+    If either are typed, then the returned semiring will also be typed.
+
+    See Also
+    --------
+    Semiring.register_anonymous
+    Semiring.register_new
+    """
+    monoid, opclass = find_opclass(monoid)
+    switched = False
+    if opclass == "BinaryOp" and monoid.monoid is not None:
+        switched = True
+        monoid = monoid.monoid
+    elif opclass != "Monoid":
+        raise TypeError(f"Expected a Monoid for the monoid argument.  Got type: {type(monoid)}")
+    binaryop, opclass = find_opclass(binaryop)
+    if opclass == "Monoid":
+        if switched:
+            raise TypeError(
+                "Got a BinaryOp for the monoid argument and a Monoid for the binaryop argument.  "
+                "Are the arguments switched?  Hint: you can do `mymonoid.binaryop` to get the "
+                "binaryop from a monoid."
+            )
+        binaryop = binaryop.binaryop
+    elif opclass != "BinaryOp":
+        raise TypeError(
+            f"Expected a BinaryOp for the binaryop argument.  Got type: {type(binaryop)}"
+        )
+    if isinstance(monoid, Monoid):
+        monoid_type = None
+    else:
+        monoid_type = monoid.type
+        monoid = monoid.parent
+    if isinstance(binaryop, BinaryOp):
+        binary_type = None
+    else:
+        binary_type = binaryop.type
+        binaryop = binaryop.parent
+    if monoid._anonymous or binaryop._anonymous:
+        rv = Semiring.register_anonymous(monoid, binaryop)
+    else:
+        name = f"{monoid.name}_{binaryop.name}".replace(".", "_")
+        rv = getattr(semiring, name, None)
+        if rv is None:
+            rv = Semiring.register_new(name, monoid, binaryop)
+        elif rv.monoid is not monoid or rv.binaryop is not binaryop:  # pragma: no cover
+            # It's not the object we expect (can this happen?)
+            rv = Semiring.register_anonymous(monoid, binaryop)
+    if binary_type is not None:
+        return rv[binary_type]
+    elif monoid_type is not None:
+        return rv[monoid_type]
+    else:
+        return rv
+
+
 # Now initialize all the things!
 UnaryOp._initialize()
 BinaryOp._initialize()
 Monoid._initialize()
 Semiring._initialize()
+
+from .agg import Aggregator, TypedAggregator  # noqa isort:skip
