@@ -1,5 +1,6 @@
 from pytest import fixture, raises
-from grblas import op, monoid, Scalar, Vector, Matrix
+
+from grblas import Matrix, Scalar, Vector, monoid, op
 from grblas.exceptions import DimensionMismatch
 
 
@@ -36,6 +37,11 @@ def test_ewise(v1, v2, A1, A2):
         (A1, A1),
     ]:
         expected = left.ewise_mult(right, monoid.times).new()
+        expr = left & right
+        assert expr.nvals == expected.nvals
+        val = expr.new(name="val")
+        assert val.name == "val"
+        assert expected.isequal((left & right).new(dtype=float))
         assert expected.isequal(monoid.times(left & right).new())
         assert expected.isequal(monoid.times[float](left & right).new())
         assert expected.isequal((left & right).new())  # use `left.ewise_mult` default op
@@ -147,8 +153,8 @@ def test_bad_ewise(s1, v1, A1, A2):
         op.minus(v1 | v1)
     with raises(TypeError):
         op.minus(v1 & v1, require_monoid=False)
-    with raises(TypeError, match="Bad types when calling binary.plus"):
-        op.plus(v1 & v1, 1)
+    # with raises(TypeError, match="Bad types when calling binary.plus"):
+    op.plus(v1 & v1, 1)  # Now okay
 
 
 def test_bad_matmul(s1, v1, A1, A2):
@@ -218,8 +224,8 @@ def test_apply_unary_bad(s1, v1):
         op.exp(s1)
     with raises(TypeError, match="Bad type when calling unary.exp"):
         op.exp(1)
-    with raises(TypeError, match="Bad type when calling unary.exp"):
-        op.exp(v1 | v1)
+    # with raises(TypeError, match="Bad type when calling unary.exp"):
+    op.exp(v1 | v1)  # Now okay
 
 
 def test_apply_binary(v1, A1):
