@@ -1,19 +1,20 @@
-import pytest
+import inspect
 import itertools
-import grblas
-import numpy as np
 import pickle
 import weakref
+
+import numpy as np
+import pytest
 from numpy.testing import assert_array_equal
-from grblas import Matrix, Vector, Scalar
-from grblas import unary, binary, monoid, semiring
-from grblas import dtypes
+
+import grblas
+from grblas import Matrix, Scalar, Vector, agg, binary, dtypes, monoid, semiring, unary
 from grblas.exceptions import (
-    IndexOutOfBound,
     DimensionMismatch,
-    OutputNotEmpty,
-    InvalidValue,
     DomainMismatch,
+    IndexOutOfBound,
+    InvalidValue,
+    OutputNotEmpty,
 )
 
 
@@ -123,6 +124,27 @@ def test_from_values():
         Matrix.from_values([0], [1, 2], [0])
 
 
+def test_from_values_scalar():
+    C = Matrix.from_values([0, 1, 3], [1, 1, 2], 7)
+    assert C.nrows == 4
+    assert C.ncols == 3
+    assert C.nvals == 3
+    assert C.dtype == dtypes.INT64
+    assert C.ss.is_iso
+    assert C.reduce_scalar(monoid.any) == 7
+
+    # iso drumps duplicates
+    C = Matrix.from_values([0, 1, 3, 0], [1, 1, 2, 1], 7)
+    assert C.nrows == 4
+    assert C.ncols == 3
+    assert C.nvals == 3
+    assert C.dtype == dtypes.INT64
+    assert C.ss.is_iso
+    assert C.reduce_scalar(monoid.any) == 7
+    with pytest.raises(ValueError, match="dup_op must be None"):
+        Matrix.from_values([0, 1, 3, 0], [1, 1, 2, 1], 7, dup_op=binary.plus)
+
+
 def test_clear(A):
     A.clear()
     assert A.nvals == 0
@@ -179,6 +201,19 @@ def test_build(A):
     assert C.isequal(Matrix.from_values([0, 0], [0, 11], [1, 1], nrows=2))
 
 
+def test_build_scalar(A):
+    assert A.nvals == 12
+    with pytest.raises(OutputNotEmpty):
+        A.ss.build_scalar([1, 5], [2, 3], 3)
+    A.clear()
+    A.ss.build_scalar([0, 6], [0, 1], 1)
+    assert A.nvals == 2
+    assert A.ss.is_iso
+    A.clear()
+    with pytest.raises(ValueError, match="lengths must match"):
+        A.ss.build_scalar([0, 6], [0, 1, 2], 1)
+
+
 def test_extract_values(A):
     rows, cols, vals = A.to_values(dtype=int)
     np.testing.assert_array_equal(rows, (0, 0, 1, 1, 2, 3, 3, 4, 5, 6, 6, 6))
@@ -374,6 +409,15 @@ def test_extract_row(A):
     assert w.isequal(result)
     w2 = A[6, [0, 2, 4]].new()
     assert w2.isequal(result)
+    with pytest.raises(TypeError):
+        # Should be list, not tuple (although tuple isn't so bad)
+        A[6, (0, 2, 4)]
+    w3 = A[6, np.array([0, 2, 4])].new()
+    assert w3.isequal(result)
+    with pytest.raises(TypeError, match="Invalid dtype"):
+        A[6, np.array([0, 2, 4], dtype=float)]
+    with pytest.raises(TypeError, match="Invalid number of dimensions"):
+        A[6, np.array([[0, 2, 4]])]
 
 
 def test_extract_column(A):
@@ -1027,6 +1071,259 @@ def test_reduce_row(A):
     assert w2.isequal(result)
 
 
+def test_reduce_agg(A):
+    result = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [5, 12, 1, 6, 7, 1, 15])
+    w1 = A.reduce_rows(agg.sum).new()
+    assert w1.isequal(result)
+    w2 = A.T.reduce_columns(agg.sum).new()
+    assert w2.isequal(result)
+
+    counts = A.dup(dtype=bool).reduce_rows(monoid.plus[int]).new()
+    w3 = A.reduce_rows(agg.count).new()
+    assert w3.isequal(counts)
+    w4 = A.T.reduce_columns(agg.count).new()
+    assert w4.isequal(counts)
+
+    Asquared = monoid.times(A & A).new()
+    squared = Asquared.reduce_rows(monoid.plus).new()
+    expected = unary.sqrt[float](squared).new()
+    w5 = A.reduce_rows(agg.hypot).new()
+    assert w5.isclose(expected)
+    w6 = A.reduce_rows(monoid.numpy.hypot[float]).new()
+    assert w6.isclose(expected)
+    w7 = Vector.new(w5.dtype, size=w5.size)
+    w7 << A.reduce_rows(agg.hypot)
+    assert w7.isclose(expected)
+
+    w8 = A.reduce_rows(agg.logaddexp).new()
+    expected = A.reduce_rows(monoid.numpy.logaddexp[float]).new()
+    assert w8.isclose(w8)
+
+    result = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 2, 9, 10, 11, 8, 4])
+    w9 = A.reduce_columns(agg.sum).new()
+    assert w9.isequal(result)
+    w10 = A.T.reduce_rows(agg.sum).new()
+    assert w10.isequal(result)
+
+    counts = A.dup(dtype=bool).reduce_columns(monoid.plus[int]).new()
+    w11 = A.reduce_columns(agg.count).new()
+    assert w11.isequal(counts)
+    w12 = A.T.reduce_rows(agg.count).new()
+    assert w12.isequal(counts)
+
+    w13 = A.reduce_rows(agg.mean).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [2.5, 6, 1, 3, 7, 1, 5])
+    assert w13.isequal(expected)
+    w14 = A.reduce_columns(agg.mean).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 2, 3, 5, 5.5, 4, 4])
+    assert w14.isequal(expected)
+
+    w15 = A.reduce_rows(agg.exists).new()
+    w16 = A.reduce_columns(agg.exists).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [1, 1, 1, 1, 1, 1, 1])
+    assert w15.isequal(expected)
+    assert w16.isequal(expected)
+
+    assert A.reduce_scalar(agg.sum).value == 47
+    assert A.reduce_scalar(agg.prod).value == 1270080
+    assert A.reduce_scalar(agg.count).value == 12
+    assert A.reduce_scalar(agg.count_nonzero).value == 12
+    assert A.reduce_scalar(agg.count_zero).value == 0
+    assert A.reduce_scalar(agg.sum_of_squares).value == 245
+    assert A.reduce_scalar(agg.hypot).new().isclose(245 ** 0.5)
+    assert A.reduce_scalar(agg.logaddexp).new().isclose(8.6071076)
+    assert A.reduce_scalar(agg.logaddexp2).new().isclose(9.2288187)
+    assert A.reduce_scalar(agg.mean).new().isclose(47 / 12)
+    assert A.reduce_scalar(agg.exists) == 1
+
+    silly = agg.Aggregator(
+        "silly",
+        composite=[agg.varp, agg.stdp],
+        finalize=lambda x, y: binary.times(x & y),
+        types=[agg.varp],
+    )
+    v1 = A.reduce_rows(agg.varp).new()
+    v2 = A.reduce_rows(agg.stdp).new()
+    assert v1.isclose(binary.times(v2 & v2).new())
+    v3 = A.reduce_rows(silly).new()
+    assert v3.isclose(binary.times(v1 & v2).new())
+
+    s1 = A.reduce_scalar(agg.varp).new()
+    s2 = A.reduce_scalar(agg.stdp).new()
+    assert s1.isclose(s2.value * s2.value)
+    s3 = A.reduce_scalar(silly).new()
+    assert s3.isclose(s1.value * s2.value)
+
+
+def test_reduce_agg_argminmax(A):
+    # reduce_rows
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [1, 6, 5, 0, 5, 2, 4])
+    w1b = A.reduce_rows(agg.argmin).new()
+    assert w1b.isequal(expected)
+    w1c = A.T.reduce_columns(agg.argmin).new()
+    assert w1c.isequal(expected)
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 4, 5, 0, 5, 2, 3])
+    w2b = A.reduce_rows(agg.argmax).new()
+    assert w2b.isequal(expected)
+    w2c = A.T.reduce_columns(agg.argmax).new()
+    assert w2c.isequal(expected)
+
+    # reduce_cols
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 0, 5, 0, 6, 2, 1])
+    w7b = A.reduce_columns(agg.argmin).new()
+    assert w7b.isequal(expected)
+    w7c = A.T.reduce_rows(agg.argmin).new()
+    assert w7c.isequal(expected)
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 0, 6, 6, 1, 4, 1])
+    w8b = A.reduce_columns(agg.argmax).new()
+    assert w8b.isequal(expected)
+    w8c = A.T.reduce_rows(agg.argmax).new()
+    assert w8c.isequal(expected)
+
+    # reduce_scalar
+    with pytest.raises(
+        ValueError, match="Aggregator argmin may not be used with Matrix.reduce_scalar"
+    ):
+        A.reduce_scalar(agg.argmin)
+
+    silly = agg.Aggregator(
+        "silly",
+        composite=[agg.argmin, agg.argmax],
+        finalize=lambda x, y: binary.plus(x & y),
+        types=[agg.argmin],
+    )
+    v1 = A.reduce_rows(agg.argmin).new()
+    v2 = A.reduce_rows(agg.argmax).new()
+    v3 = A.reduce_rows(silly).new()
+    assert v3.isequal(binary.plus(v1 & v2).new())
+
+    v1 = A.reduce_columns(agg.argmin).new()
+    v2 = A.reduce_columns(agg.argmax).new()
+    v3 = A.reduce_columns(silly).new()
+    assert v3.isequal(binary.plus(v1 & v2).new())
+
+    with pytest.raises(ValueError, match="Aggregator"):
+        A.reduce_scalar(silly).new()
+
+
+def test_reduce_agg_firstlast(A):
+    # reduce_rows
+    w1 = A.reduce_rows(agg.first).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [2, 8, 1, 3, 7, 1, 5])
+    assert w1.isequal(expected)
+    w1b = A.T.reduce_columns(agg.first).new()
+    assert w1b.isequal(expected)
+    w2 = A.reduce_rows(agg.last).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 4, 1, 3, 7, 1, 3])
+    assert w2.isequal(expected)
+    w2b = A.T.reduce_columns(agg.last).new()
+    assert w2b.isequal(expected)
+
+    # reduce_columns
+    w3 = A.reduce_columns(agg.first).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 2, 3, 3, 8, 1, 4])
+    assert w3.isequal(expected)
+    w3b = A.T.reduce_rows(agg.first).new()
+    assert w3b.isequal(expected)
+    w4 = A.reduce_columns(agg.last).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 2, 5, 7, 3, 7, 4])
+    assert w4.isequal(expected)
+    w4b = A.T.reduce_rows(agg.last).new()
+    assert w4b.isequal(expected)
+
+    # reduce_scalar
+    w5 = A.reduce_scalar(agg.first).new()
+    assert w5 == 2
+    w6 = A.reduce_scalar(agg.last).new()
+    assert w6 == 3
+    B = Matrix.new(float, nrows=2, ncols=3)
+    assert B.reduce_scalar(agg.first).value is None
+    assert B.reduce_scalar(agg.last).value is None
+    w7 = B.reduce_rows(agg.first).new()
+    assert w7.isequal(Vector.new(float, size=B.nrows))
+    w8 = B.reduce_columns(agg.last).new()
+    assert w8.isequal(Vector.new(float, size=B.ncols))
+
+    silly = agg.Aggregator(
+        "silly",
+        composite=[agg.first, agg.last],
+        finalize=lambda x, y: binary.plus(x & y),
+        types=[agg.first],
+    )
+    v1 = A.reduce_rows(agg.first).new()
+    v2 = A.reduce_rows(agg.last).new()
+    v3 = A.reduce_rows(silly).new()
+    assert v3.isequal(binary.plus(v1 & v2).new())
+
+    s1 = A.reduce_scalar(agg.first).new()
+    s2 = A.reduce_scalar(agg.last).new()
+    s3 = A.reduce_scalar(silly).new()
+    assert s3.isequal(s1.value + s2.value)
+
+
+def test_reduce_agg_firstlast_index(A):
+    # reduce_rows
+    w1 = A.reduce_rows(agg.first_index).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [1, 4, 5, 0, 5, 2, 2])
+    assert w1.isequal(expected)
+    w1b = A.T.reduce_columns(agg.first_index).new()
+    assert w1b.isequal(expected)
+    w2 = A.reduce_rows(agg.last_index).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 6, 5, 2, 5, 2, 4])
+    assert w2.isequal(expected)
+    w2b = A.T.reduce_columns(agg.last_index).new()
+    assert w2b.isequal(expected)
+
+    # reduce_columns
+    w3 = A.reduce_columns(agg.first_index).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 0, 3, 0, 1, 2, 1])
+    assert w3.isequal(expected)
+    w3b = A.T.reduce_rows(agg.first_index).new()
+    assert w3b.isequal(expected)
+    w4 = A.reduce_columns(agg.last_index).new()
+    expected = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [3, 0, 6, 6, 6, 4, 1])
+    assert w4.isequal(expected)
+    w4b = A.T.reduce_rows(agg.last_index).new()
+    assert w4b.isequal(expected)
+
+    # reduce_scalar
+    with pytest.raises(ValueError, match="Aggregator first_index may not"):
+        A.reduce_scalar(agg.first_index).new()
+    with pytest.raises(ValueError, match="Aggregator last_index may not"):
+        A.reduce_scalar(agg.last_index).new()
+
+    silly = agg.Aggregator(
+        "silly",
+        composite=[agg.first_index, agg.last_index],
+        finalize=lambda x, y: binary.plus(x & y),
+        types=[agg.first_index],
+    )
+    v1 = A.reduce_rows(agg.first_index).new()
+    v2 = A.reduce_rows(agg.last_index).new()
+    v3 = A.reduce_rows(silly).new()
+    assert v3.isequal(binary.plus(v1 & v2).new())
+
+    with pytest.raises(ValueError, match="Aggregator"):
+        A.reduce_scalar(silly).new()
+
+
+def test_reduce_agg_empty():
+    A = Matrix.new("UINT8", nrows=3, ncols=4)
+    for B in [A, A.T]:
+        ve = Vector.new(bool, size=B.nrows)
+        we = Vector.new(bool, size=B.ncols)
+        for attr, aggr in vars(agg).items():
+            if not isinstance(aggr, agg.Aggregator):
+                continue
+            v = B.reduce_rows(aggr).new()
+            assert ve.isequal(v)
+            w = B.reduce_columns(aggr).new()
+            assert we.isequal(w)
+            if attr not in {"argmin", "argmax", "first_index", "last_index"}:
+                s = B.reduce_scalar(aggr).new()
+                assert s.value is None
+
+
 def test_reduce_row_udf(A):
     result = Vector.from_values([0, 1, 2, 3, 4, 5, 6], [5, 12, 1, 6, 7, 1, 15])
     binop = grblas.operator.BinaryOp.register_anonymous(lambda x, y: x + y)
@@ -1237,8 +1534,9 @@ def test_transpose_exceptional():
         B.T[1, 0]() << 10
     with pytest.raises(TypeError, match="not callable"):
         B.T()[1, 0] << 10
-    with pytest.raises(AttributeError):
-        B.T.dup()  # should use new instead
+    # with pytest.raises(AttributeError):
+    # should use new instead--Now okay.
+    assert B.T.dup().isequal(B.T.new())
     # Not exceptional, but while we're here...
     C = B.T.new(mask=A.V)
     D = B.T.new()
@@ -1518,6 +1816,39 @@ def test_import_export(A, do_iso, methods):
         assert A8.isequal(A)
         assert A8.ss.is_iso is do_iso
 
+    A9 = A.dup()
+    d = getattr(A9.ss, out_method)("coo", sort=True)
+    if do_iso:
+        assert d["is_iso"] is True
+        assert_array_equal(d["values"], [1])
+    else:
+        assert "is_iso" not in d
+    assert d["nrows"] == 7
+    assert d["ncols"] == 7
+    assert d["rows"].shape == (12,)
+    assert d["cols"].shape == (12,)
+    assert d["sorted_rows"]
+    assert "sorted_cols" not in d
+    assert_array_equal(d["rows"], [0, 0, 1, 1, 2, 3, 3, 4, 5, 6, 6, 6])
+    assert_array_equal(d["cols"], [1, 3, 4, 6, 5, 0, 2, 5, 2, 2, 3, 4])
+
+    if do_iso:
+        assert d["values"].shape == (1,)
+    else:
+        assert d["values"].shape == (12,)
+    if in_method == "import":
+        B8 = Matrix.ss.import_any(**d)
+        assert B8.isequal(A)
+        assert B8.ss.is_iso is do_iso
+        del d["rows"]
+        del d["format"]
+        with pytest.raises(ValueError, match="coo requires both"):
+            Matrix.ss.import_any(**d)
+    else:
+        A9.ss.pack_any(**d)
+        assert A9.isequal(A)
+        assert A9.ss.is_iso is do_iso
+
     C = Matrix.from_values([0, 0, 1, 1], [0, 1, 0, 1], [1, 2, 3, 4])
     if do_iso:
         C(C.S) << 1
@@ -1675,6 +2006,14 @@ def test_import_export_empty():
     with pytest.raises(InvalidValue):
         A.dup().ss.export("fullc")
 
+    A7 = A.dup()
+    d = A7.ss.export("coo")
+    assert d["nrows"] == 2
+    assert d["ncols"] == 3
+    assert len(d["rows"]) == 0
+    assert len(d["cols"]) == 0
+    assert len(d["values"]) == 0
+
     # if we give the same value, make sure it's copied
     for format, key1, key2 in [
         ("csr", "values", "col_indices"),
@@ -1683,6 +2022,7 @@ def test_import_export_empty():
         ("hypercsc", "values", "row_indices"),
         ("bitmapr", "values", "bitmap"),
         ("bitmapc", "values", "bitmap"),
+        ("coo", "values", "rows"),
     ]:
         # No assertions here, but code coverage should be "good enough"
         d = A.ss.export(format, raw=True)
@@ -1700,7 +2040,7 @@ def test_import_export_auto(A, do_iso, methods):
         A(A.S) << 1
     A_orig = A.dup()
     out_method, in_method = methods
-    for format in ["csr", "csc", "hypercsr", "hypercsc", "bitmapr", "bitmapc"]:
+    for format in ["csr", "csc", "hypercsr", "hypercsc", "bitmapr", "bitmapc", "coo"]:
         for (
             sort,
             raw,
@@ -2027,3 +2367,103 @@ def test_concat(A):
 
 def test_nbytes(A):
     assert A.ss.nbytes > 0
+
+
+def test_auto(A, v):
+    expected = binary.times(A & A).new()
+    for expr in [(A & A), binary.times(A & A)]:
+        assert expr.dtype == expected.dtype
+        assert expr.nrows == expected.nrows
+        assert expr.ncols == expected.ncols
+        assert expr.shape == expected.shape
+        assert expr.nvals == expected.nvals
+        assert expr.isclose(expected)
+        assert expected.isclose(expr)
+        assert expr.isequal(expected)
+        assert expected.isequal(expr)
+        assert expr.mxv(v).isequal(expected.mxv(v))
+        assert expected.T.mxv(v).isequal(expr.T.mxv(v))
+        for method in [
+            "ewise_add",
+            "ewise_mult",
+            "mxm",
+            "__matmul__",
+            "__and__",
+            "__or__",
+            "kronecker",
+        ]:
+            val1 = getattr(expected, method)(expected).new()
+            val2 = getattr(expected, method)(expr)
+            val3 = getattr(expr, method)(expected)
+            val4 = getattr(expr, method)(expr)
+            assert val1.isequal(val2)
+            assert val1.isequal(val3)
+            assert val1.isequal(val4)
+        for method in ["reduce_rows", "reduce_columns", "reduce_scalar"]:
+            s1 = getattr(expected, method)().new()
+            s2 = getattr(expr, method)()
+            assert s1.isequal(s2.new())
+            assert s1.isequal(s2)
+
+    expected = semiring.plus_times(A @ v).new()
+    for expr in [(A @ v), (v @ A.T), semiring.plus_times(A @ v)]:
+        assert expr.vxm(A).isequal(expected.vxm(A))
+        assert expr.vxm(A).new(mask=expr.S).isequal(expected.vxm(A).new(mask=expected.S))
+        assert expr.vxm(A).new(mask=expr.V).isequal(expected.vxm(A).new(mask=expected.V))
+
+
+def test_auto_assign(A):
+    expected = A.dup()
+    B = A[1:4, 1:4].new()
+    expr = B & B
+    expected[:3, :3] = expr.new()
+    A[:3, :3] = expr
+    assert expected.isequal(A)
+    with pytest.raises(TypeError):
+        # Not yet supported, but we could!
+        A[:3, :3] = A[1:4, 1:4]
+    v = A[2:5, 5].new()
+    expr = v & v
+    A[:3, 4] << expr
+    expected[:3, 4] << expr.new()
+    assert expected.isequal(A)
+
+
+def test_expr_is_like_matrix(A):
+    attrs = {attr for attr, val in inspect.getmembers(A)}
+    expr_attrs = {attr for attr, val in inspect.getmembers(binary.times(A & A))}
+    infix_attrs = {attr for attr, val in inspect.getmembers(A & A)}
+    transposed_attrs = {attr for attr, val in inspect.getmembers(A.T)}
+    # Should we make any of these raise informative errors?
+    expected = {
+        "__call__",
+        "__del__",
+        "__delitem__",
+        "__lshift__",
+        "__setitem__",
+        "_assign_element",
+        "_delete_element",
+        "_deserialize",
+        "_extract_element",
+        "_name_counter",
+        "_prep_for_assign",
+        "_prep_for_extract",
+        "_update",
+        "build",
+        "clear",
+        "from_pygraphblas",
+        "from_values",
+        "resize",
+        "update",
+    }
+    assert attrs - expr_attrs == expected
+    assert attrs - infix_attrs == expected | {
+        "_expect_op",
+        "_expect_type",
+    }
+    # TransposedMatrix is used differently than other expressions,
+    # so maybe it shouldn't support everything.
+    assert attrs - transposed_attrs == (expected | {"S", "V", "ss", "to_pygraphblas", "wait"}) - {
+        "_prep_for_extract",
+        "_extract_element",
+    }
