@@ -1,11 +1,21 @@
-import pytest
 import numpy as np
+import pytest
+
 import grblas
-from grblas import lib
-from grblas import unary, binary, monoid, semiring
-from grblas import dtypes, operator, exceptions
-from grblas import Vector, Matrix
-from grblas.operator import UnaryOp, BinaryOp, Monoid, Semiring
+from grblas import (
+    Matrix,
+    Vector,
+    agg,
+    binary,
+    dtypes,
+    exceptions,
+    lib,
+    monoid,
+    operator,
+    semiring,
+    unary,
+)
+from grblas.operator import BinaryOp, Monoid, Semiring, UnaryOp, get_semiring
 
 
 def orig_types(op):
@@ -44,6 +54,17 @@ def test_semiring():
     assert orig_types(semiring.min_firsti) == {"INT32", "INT64"}
 
 
+def test_agg():
+    assert repr(agg.count) == "agg.count"
+    assert repr(agg.count["INT32"]) == "agg.count[INT32]"
+    assert "INT64" in agg.sum_of_inverses
+    assert agg.sum_of_inverses["INT64"].return_type == "FP64"
+    assert "BOOL" not in agg.sum_of_inverses
+    with pytest.raises(KeyError, match="BOOL"):
+        agg.sum_of_inverses["BOOL"]
+    assert agg.varp["INT64"].return_type == "FP64"
+
+
 def test_find_opclass_unaryop():
     assert operator.find_opclass(unary.minv)[1] == "UnaryOp"
     # assert operator.find_opclass(lib.GrB_MINV_INT64)[1] == 'UnaryOp'
@@ -724,3 +745,39 @@ def test_div_semirings():
     result = A1.T.mxm(A2, semiring.plus_floordiv).new()
     assert result[0, 0].value == -3
     assert result.dtype == dtypes.INT64
+
+
+def test_get_semiring():
+    sr = get_semiring(monoid.plus, binary.times)
+    assert sr is semiring.plus_times
+    # Be somewhat forgiving
+    sr = get_semiring(monoid.plus, monoid.times)
+    assert sr is semiring.plus_times
+    sr = get_semiring(binary.plus, binary.times)
+    assert sr is semiring.plus_times
+    # But not if switched
+    with pytest.raises(TypeError, match="switch"):
+        get_semiring(binary.plus, monoid.times)
+
+    def myplus(x, y):
+        return x + y
+
+    binop = BinaryOp.register_anonymous(myplus, name="myplus")
+    st = get_semiring(monoid.plus, binop)
+    assert st.monoid is monoid.plus
+    assert st.binaryop is binop
+
+    binop = BinaryOp.register_new("myplus", myplus)
+    assert binop is binary.myplus
+    st = get_semiring(monoid.plus, binop)
+    assert st.monoid is monoid.plus
+    assert st.binaryop is binop
+
+    with pytest.raises(TypeError, match="Monoid"):
+        get_semiring(None, binary.times)
+    with pytest.raises(TypeError, match="Binary"):
+        get_semiring(monoid.plus, None)
+
+    sr = get_semiring(monoid.plus, binary.numpy.copysign)
+    assert sr.monoid is monoid.plus
+    assert sr.binaryop is binary.numpy.copysign
