@@ -1,14 +1,20 @@
-import pytest
+import inspect
 import itertools
-import numpy as np
-import grblas
 import pickle
 import weakref
+
+import numpy as np
+import pytest
 from numpy.testing import assert_array_equal
-from grblas import Matrix, Vector, Scalar
-from grblas import unary, binary, monoid, semiring
-from grblas import dtypes
-from grblas.exceptions import IndexOutOfBound, OutputNotEmpty, DimensionMismatch, InvalidValue
+
+import grblas
+from grblas import Matrix, Scalar, Vector, agg, binary, dtypes, monoid, semiring, unary
+from grblas.exceptions import (
+    DimensionMismatch,
+    IndexOutOfBound,
+    InvalidValue,
+    OutputNotEmpty,
+)
 
 
 @pytest.fixture
@@ -108,6 +114,24 @@ def test_from_values():
         Vector.from_values([0], [1, 2])
 
 
+def test_from_values_scalar():
+    u = Vector.from_values([0, 1, 3], 7)
+    assert u.size == 4
+    assert u.nvals == 3
+    assert u.dtype == dtypes.INT64
+    assert u.ss.is_iso
+    assert u.reduce(monoid.any) == 7
+
+    # ignore duplicate indices; iso trumps duplicates!
+    u = Vector.from_values([0, 1, 1, 3], 7)
+    assert u.size == 4
+    assert u.nvals == 3
+    assert u.ss.is_iso
+    assert u.reduce(monoid.any) == 7
+    with pytest.raises(ValueError, match="dup_op must be None"):
+        Vector.from_values([0, 1, 1, 3], 7, dup_op=binary.plus)
+
+
 def test_clear(v):
     v.clear()
     assert v.nvals == 0
@@ -153,6 +177,15 @@ def test_build(v):
     assert w.isequal(Vector.from_values([0, 11], [1, 1]))
 
 
+def test_build_scalar(v):
+    with pytest.raises(OutputNotEmpty):
+        v.ss.build_scalar([1, 5], 3)
+    v.clear()
+    v.ss.build_scalar([1, 5], 3)
+    assert v.nvals == 2
+    assert v.ss.is_iso
+
+
 def test_extract_values(v):
     idx, vals = v.to_values()
     np.testing.assert_array_equal(idx, (1, 3, 4, 6))
@@ -192,6 +225,10 @@ def test_extract_input_mask():
 def test_extract_element(v):
     assert v[1].value == 1
     assert v[6].new() == 0
+    with pytest.raises(TypeError, match="Invalid type for index"):
+        v[object()]
+    with pytest.raises(IndexError):
+        v[100]
 
 
 def test_set_element(v):
@@ -580,6 +617,104 @@ def test_reduce(v):
         b1.reduce()
 
 
+def test_reduce_agg(v):
+    s = v.reduce(agg.sum).new()
+    assert s.dtype == "INT64"
+    assert s == 4
+    s = v.reduce(agg.sum[float]).new()
+    assert s.dtype == "FP64"
+    assert s == 4
+    assert v.reduce(agg.prod) == 0
+    assert v.reduce(agg.count) == 4
+    assert v.reduce(agg.count_nonzero) == 3
+    assert v.reduce(agg.count_zero) == 1
+    assert v.reduce(agg.sum_of_squares) == 6
+    assert v.reduce(agg.hypot).new().isclose(6 ** 0.5)
+    assert v.reduce(agg.logaddexp).new().isclose(np.log(1 + 2 * np.e + np.e ** 2))
+    assert v.reduce(agg.logaddexp2).new().isclose(np.log2(9))
+    assert v.reduce(agg.mean) == 1
+    assert v.reduce(agg.peak_to_peak) == 2
+    assert v.reduce(agg.varp).new().isclose(0.5)
+    assert v.reduce(agg.vars).new().isclose(2 / 3)
+    assert v.reduce(agg.stdp).new().isclose(0.5 ** 0.5)
+    assert v.reduce(agg.stds).new().isclose((2 / 3) ** 0.5)
+    assert v.reduce(agg.L0norm) == 3
+    assert v.reduce(agg.L1norm) == 4
+    assert v.reduce(agg.L2norm).new().isclose(6 ** 0.5)
+    assert v.reduce(agg.Linfnorm) == 2
+    assert v.reduce(agg.exists) == 1
+    w = binary.plus(v, 1).new()
+    assert w.reduce(agg.geometric_mean).new().isclose(12 ** 0.25)
+    assert w.reduce(agg.harmonic_mean).new().isclose(12 / 7)
+
+    silly = agg.Aggregator(
+        "silly",
+        composite=[agg.varp, agg.stdp],
+        finalize=lambda x, y: binary.times(x & y),
+        types=[agg.varp],
+    )
+    s = v.reduce(silly).new()
+    assert s.isclose(0.5 ** 1.5)
+
+    s = Vector.new(int, size=5).reduce(silly).new()
+    assert s.is_empty
+
+
+def test_reduce_agg_argminmax(v):
+    assert v.reduce(agg.argmin).value == 6
+    assert v.reduce(agg.argmax).value == 4
+
+    silly = agg.Aggregator(
+        "silly",
+        composite=[agg.argmin, agg.argmax],
+        finalize=lambda x, y: binary.plus(x & y),
+        types=[agg.argmin],
+    )
+    s = v.reduce(silly).new()
+    assert s == 10
+
+
+def test_reduce_agg_firstlast(v):
+    empty = Vector.new(int, size=4)
+    assert empty.reduce(agg.first).value is None
+    assert empty.reduce(agg.last).value is None
+
+    assert v.reduce(agg.first).value == 1
+    assert v.reduce(agg.last).value == 0
+
+    silly = agg.Aggregator(
+        "silly",
+        composite=[agg.first, agg.last],
+        finalize=lambda x, y: binary.plus(x & y),
+        types=[agg.first],
+    )
+    s = v.reduce(silly).new()
+    assert s == 1
+
+
+def test_reduce_agg_firstlast_index(v):
+    assert v.reduce(agg.first_index).value == 1
+    assert v.reduce(agg.last_index).value == 6
+
+    silly = agg.Aggregator(
+        "silly",
+        composite=[agg.first_index, agg.last_index],
+        finalize=lambda x, y: binary.plus(x & y),
+        types=[agg.first_index],
+    )
+    s = v.reduce(silly).new()
+    assert s == 7
+
+
+def test_reduce_agg_empty():
+    v = Vector.new("UINT8", size=3)
+    for attr, aggr in vars(agg).items():
+        if not isinstance(aggr, agg.Aggregator):
+            continue
+        s = v.reduce(aggr).new()
+        assert s.value is None
+
+
 def test_reduce_coerce_dtype(v):
     assert v.dtype == dtypes.INT64
     s = v.reduce().new(dtype=float)
@@ -666,6 +801,8 @@ def test_accum_must_be_binaryop(v):
 def test_mask_must_be_value_or_structure(v):
     with pytest.raises(TypeError):
         v(mask=v) << v.ewise_mult(v)
+    with pytest.raises(TypeError):
+        v(mask=object()) << v.ewise_mult(v)
 
 
 def test_incompatible_shapes(A, v):
@@ -1024,3 +1161,121 @@ def test_outer(v):
     expected = C.mxm(R).new()
     result = v.outer(v).new()
     assert result.isequal(expected)
+    result = v.outer(v, monoid.times).new()
+    assert result.isequal(expected)
+
+
+def test_auto(v):
+    expected = binary.times(v & v).new()
+    assert 0 not in expected
+    assert 1 in expected
+    for expr in [(v & v), binary.times(v & v)]:
+        assert expr.size == expected.size
+        assert expr.dtype == expected.dtype
+        assert expr.shape == expected.shape
+        assert expr.nvals == expected.nvals
+        assert expr._nvals == expected._nvals
+        assert expr.isclose(expected)
+        val = expr.new(name="val")
+        assert val.name == "val"
+        assert expr._value is None
+        assert expected.isclose(expr)
+        assert expr.isequal(expected)
+        assert expected.isequal(expr)
+        assert 0 not in expr
+        assert 1 in expr
+        assert expr[0].value == expected[0].value
+        assert expr[1].value == expected[1].value
+        assert list(expr) == list(expected)
+        k1, v1 = expected.to_values()
+        k2, v2 = expr.to_values()
+        assert_array_equal(k1, k2)
+        assert_array_equal(v1, v2)
+        unary.sqrt(expected).isequal(unary.sqrt(expr))
+        assert expr.gb_obj is not expected.gb_obj
+        assert expr.gb_obj is expr.gb_obj
+        assert expr.name != expected.name
+        expr.name = "new name"
+        assert expr.name == "new name"
+        # Probably no need for _name_html or _carg
+        assert expr._name_html != expected._name_html
+        assert expr._carg != expected._carg
+        for method in [
+            "ewise_add",
+            "ewise_mult",
+            "inner",
+            "outer",
+            "__matmul__",
+            "__and__",
+            "__or__",
+            "__rmatmul__",
+            "__rand__",
+            "__ror__",
+        ]:
+            val1 = getattr(expected, method)(expected).new()
+            val2 = getattr(expected, method)(expr)
+            val3 = getattr(expr, method)(expected)
+            val4 = getattr(expr, method)(expr)
+            assert val1.isequal(val2)
+            assert val1.isequal(val3)
+            assert val1.isequal(val4)
+            assert val1.isequal(val2.new())
+            assert val1.isequal(val3.new())
+            assert val1.isequal(val4.new())
+        s1 = expected.reduce().new()
+        s2 = expr.reduce()
+        assert s1.isequal(s2.new())
+        assert s1.isequal(s2)
+        assert s1.is_empty == s2.is_empty
+        assert -s1 == -s2
+        assert complex(s1) == complex(s2)
+        assert_array_equal(np.array([s1]), np.array([s2]))
+    w = v.dup()
+    expected = v.dup()
+    expected(binary.plus) << (w & w).new()
+    w(binary.plus) << (w & w)
+
+
+def test_auto_assign(v):
+    expected = v.dup()
+    w = v[1:4].new()
+    expr = w & w
+    expected[:3] = expr.new()
+    v[:3] = expr
+    assert expected.isequal(v)
+    with pytest.raises(TypeError):
+        # Not yet supported, but we could!
+        v[:3] = v[1:4]
+
+
+def test_expr_is_like_vector(v):
+    attrs = {attr for attr, val in inspect.getmembers(v)}
+    expr_attrs = {attr for attr, val in inspect.getmembers(binary.times(v & v))}
+    infix_attrs = {attr for attr, val in inspect.getmembers(v & v)}
+    # Should we make any of these raise informative errors?
+    expected = {
+        "__call__",
+        "__del__",
+        "__delitem__",
+        "__lshift__",
+        "__setitem__",
+        "_assign_element",
+        "_delete_element",
+        "_deserialize",
+        "_extract_element",
+        "_name_counter",
+        "_prep_for_assign",
+        "_prep_for_extract",
+        "_update",
+        "build",
+        "clear",
+        "from_pygraphblas",
+        "from_values",
+        "resize",
+        "update",
+    }
+    assert attrs - expr_attrs == expected
+    assert attrs - infix_attrs == expected | {
+        "_expect_op",
+        "_expect_type",
+    }
